@@ -11,6 +11,7 @@
 #' @importFrom ggplot2 ggplot aes geom_line geom_col geom_rug labs
 #' @importFrom rlang sym
 #' @importFrom recipes bake
+#' @param data Character string specifying which data to use: \code{"train"} (default) or \code{"test"}.
 #' @export
 #' @examples
 #' \dontrun{
@@ -20,7 +21,7 @@
 #' model <- fastml(data = iris, label = "Species")
 #' explain_ale(model, feature = "Sepal.Length")
 #' }
-explain_ale <- function(object, feature, ...) {
+explain_ale <- function(object, feature, data = c("train", "test"), ...) {
   if (!inherits(object, "fastml")) {
     stop("The input must be a 'fastml' object.")
   }
@@ -31,7 +32,8 @@ explain_ale <- function(object, feature, ...) {
     stop("The 'iml' package is required for ALE explanations.")
   }
 
-  prep <- fastml_prepare_explainer_inputs(object)
+  data <- match.arg(data)
+  prep <- fastml_prepare_explainer_inputs(object, data = data)
   train_data <- prep$train_data
   if (is.null(train_data) || !(prep$label %in% names(train_data))) {
     stop("Training data not available for ALE.")
@@ -39,19 +41,50 @@ explain_ale <- function(object, feature, ...) {
   x <- prep$x_raw
   y <- prep$y_raw
 
-  # Convert factor/character targets to numeric (0/1) so iml can compute residuals
-  positive_class <- prep$positive_class
+  # Validate that user-provided feature exists in the data
+  available_features <- colnames(x)
+  if (!(feature %in% available_features)) {
+    stop(
+      sprintf(
+        "Feature '%s' not found in data.\nAvailable features: %s%s\nNote: feature names must match exactly (case-sensitive).",
+        feature,
+        paste(utils::head(available_features, 10), collapse = ", "),
+        if (length(available_features) > 10) " ..." else ""
+      ),
+      call. = FALSE
+    )
+  }
+
+  # Handle classification targets for iml
+
+  # Determine if this is multiclass (>2 classes)
+  is_multiclass <- FALSE
+  positive_class <- NULL
   if (is.factor(y) || is.character(y)) {
     y_factor <- if (is.factor(y)) y else factor(y)
-    if (is.null(positive_class) || !(positive_class %in% levels(y_factor))) {
-      positive_class <- levels(y_factor)[1]
+    n_classes <- length(levels(y_factor))
+    is_multiclass <- n_classes > 2
+
+    if (is_multiclass) {
+      # Multiclass: keep y as factor, iml will handle it properly
+      # when predict function returns all class probabilities
+      y <- y_factor
+    } else {
+      # Binary classification: convert to numeric (0/1) for iml
+      # Use resolve_positive_class to respect event_class settings from fastml()
+      positive_class <- resolve_positive_class(prep, levels(y_factor))
+      y <- as.numeric(y_factor == positive_class)
     }
-    y <- as.numeric(y_factor == positive_class)
+  } else {
+    positive_class <- prep$positive_class
   }
 
   parsnip_fit <- prep$fits[[1]]
 
-  # Custom predict function returning a numeric probability vector for the positive class
+  # Custom predict function
+  # For multiclass: returns data.frame with all class probabilities
+
+  # For binary: returns numeric probability vector for the positive class
   predict_fun <- function(model, newdata) {
     newdata_processed <- tryCatch(
       {
@@ -70,7 +103,13 @@ explain_ale <- function(object, feature, ...) {
       error = function(e) NULL
     )
     if (!is.null(prob)) {
-      # Prefer column that matches the positive class; otherwise fall back to first numeric column
+      # Multiclass: return full probability data.frame (iml handles this)
+      if (is_multiclass) {
+        # Clean column names: remove .pred_ prefix
+        colnames(prob) <- sub("^\\.pred_", "", colnames(prob))
+        return(as.data.frame(prob))
+      }
+      # Binary: return probability for positive class
       prob_col <- paste0(".pred_", positive_class)
       if (!is.null(positive_class) && prob_col %in% names(prob)) {
         return(as.numeric(prob[[prob_col]]))
